// Basic floating point vector operations.
// Written by Nils Liaaen Corneliusen 2017.
// License: CC0 1.0 Universal (CC0 1.0) Public Domain Dedication license
// Please refer to the article at http://www.ignorantus.com/pages/tilegx_integer_raytracer/
// for more information.
#ifndef VEC3H
#define VEC3H

#include <stdint.h>

typedef struct
{
    float x;
    float y;
    float z;
} vec3;


static inline vec3 vec3_set( float x, float y, float z )
{
    vec3 r;

    r.x = x;
    r.y = y;
    r.z = z;

    return r;
}

static inline vec3 vec3_add( vec3 a, vec3 b )
{
    vec3 r;

    r.x = a.x + b.x;
    r.y = a.y + b.y;
    r.z = a.z + b.z;

    return r;
}

static inline vec3 vec3_sub( vec3 a, vec3 b )
{
    vec3 r;

    r.x = a.x - b.x;
    r.y = a.y - b.y;
    r.z = a.z - b.z;

    return r;
}

static inline vec3 vec3_mix( vec3 a, vec3 b, float f )
{
    vec3 out;

    out.x = a.x*(1-f) + b.x*f;
    out.y = a.y*(1-f) + b.y*f;
    out.z = a.z*(1-f) + b.z*f;

    return out;
}

static inline float norm2( vec3 a )
{
    return a.x * a.x + a.y * a.y + a.z * a.z;
}

#endif
